defmodule LinkShortener.Links do
  use Ecto.Schema
  import Ecto.Changeset
  import Ecto.Query
  alias LinkShortener.Repo
  alias LinkShortener.Links.Link
  alias LinkShortener.Generators.LinkWithRandomShorten, as: LinkGenerator
  alias LinkShortener.Accounts.User

  def new_one(), do: Link.changeset(%Link{}, %{})

  @doc """
  Create a Link with random shorten with the length.

  ## Options
    • :is_atom_based (boolean, true by default) - if a map is
      atom-based, then a key for shorten will be :shorten, else
      "shorten".
    • :length (integer, 10 by default) - the length for
      the shorten.
    • :generator (function) - the shorten is generated by the
      generator function.
      LinkShortener.Generators.LinkWithRandomShorten.generate_one/2 by
      default.
  """
  @spec create_one(map(), [is_atom_based: boolean(), length: integer(), generator: function()]) ::
          [map()]
  def create_one(attrs, opts \\ []) do
    is_atom_based = Keyword.get(opts, :is_atom_based, true)
    length = Keyword.get(opts, :length, 10)
    generator = Keyword.get(opts, :generator, &LinkGenerator.generate_one/2)

    generator.(attrs, is_atom_based: is_atom_based, length: length)
    |> insert_one()
  end

  def insert_one(attrs) do
    %Link{}
    |> Link.changeset(attrs)
    |> Repo.insert()
  end

  def get_one!(id) do
    Repo.get!(Link, id)
  end

  def get_one_by!(attrs) do
    Repo.get_by!(Link, attrs)
  end

  def get_one_by_shorten!(shorten) do
    get_one_by!(%{shorten: shorten})
  end

  def get_one_by(attrs) do
    Repo.get_by(Link, attrs)
  end

  def get_one_by_shorten(shorten) do
    get_one_by(%{shorten: shorten})
  end

  def get_all() do
    from(Link)
    |> Repo.all()
  end

  def get_all(opts) do
    from(Link)
    |> Repo.all()
  end

  def get_all_by_user(%User{id: user_id} = user) do
    from(Link)
    |> where([l], l.user_id == ^user_id)
    |> Repo.all()
  end

  def edit_one(%Link{} = link, attrs \\ %{}) do
    link
    |> Link.changeset(attrs)
  end

  def update_one(%Link{} = link, changes) do
    link
    |> Link.changeset(changes)
    |> Repo.update()
  end

  def delete_one(%Link{} = link), do: Repo.delete(link)
end
