defmodule LinkShortenerWeb.LinkLive.FormComponent do
  use LinkShortenerWeb, :live_component

  alias LinkShortener.Links

  attr :is_checked, :boolean, default: true

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        <%= @title %>
        <:subtitle>Use this form to manage link records.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="link-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:name]} type="text" label="Name" placeholder="Enter a name here" />
        <.input field={@form[:url]} type="text" label="Url" placeholder="Enter an url here" />

        <%= if @is_checked and @action == :new do %>
          <.label>Shorten</.label>
        <% end %>

        <%= unless @is_checked and @action != :edit do %>
          <.input
            field={@form[:shorten]}
            type="text"
            label="Shorten"
            placeholder="Enter a shorten here"
          />
        <% end %>

        <%= if @action == :new do %>
          <label class="inline-flex items-center cursor-pointer">
            <.input
              id="random-toggle"
              field={@form[:toggle]}
              type="checkbox"
              label="Random shorten"
              checked={@is_checked}
              phx-click="toggle"
              phx-target={@myself}
              phx-value-checked={!@is_checked}
              class="peer sr-only"
            />
          </label>
        <% end %>

        <:actions>
          <.button phx-disable-with="Saving...">Save Link</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(%{link: link} = assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_new(:form, fn ->
       to_form(Links.edit_one(link))
     end)}
  end

  @impl true
  def handle_event("validate", %{"link" => link_params}, socket) do
    link_params = with_user_id(link_params, socket.assigns.current_user)
    changeset = Links.edit_one(socket.assigns.link, link_params)
    {:noreply, assign(socket, form: to_form(changeset, action: :validate))}
  end

  @impl true
  def handle_event("toggle", params, socket) do
    is_checked = Map.get(params, "value", "false") == "true"
    {:noreply, assign(socket, :is_checked, is_checked)}
  end

  def handle_event("save", %{"link" => link_params}, socket) do
    save_link(socket, socket.assigns.action, link_params)
  end

  defp save_link(socket, :edit, link_params) do
    case Links.update_one(socket.assigns.link, link_params) do
      {:ok, link} ->
        notify_parent({:saved, link})

        {:noreply,
         socket
         |> put_flash(:info, "Link updated successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, form: to_form(changeset))}
    end
  end

  defp save_link(socket, :new, link_params) do
    {is_random_string, link_params} = Map.pop(link_params, "toggle")
    is_random = is_random_string == "true"

    link_params =
      link_params
      |> with_user_id(socket.assigns.current_user)

    case create_link(link_params, is_random) do
      {:ok, link} ->
        notify_parent({:saved, link})

        {:noreply,
         socket
         |> put_flash(:info, "Link created successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, form: to_form(changeset))}
    end
  end

  defp create_link(link_params, is_shorten_random) do
    if is_shorten_random do
      Links.create_one(link_params, is_atom_based: false)
    else
      Links.insert_one(link_params)
    end
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp with_user_id(params, current_user) do
    if Map.has_key?(params, "user_id") do
      params
    else
      params
      |> Map.put("user_id", current_user.id)
    end
  end
end
