defmodule LinkShortenerWeb.Api.V1.AccountsControllerTest do
  use LinkShortenerWeb.ConnCase

  import LinkShortener.AccountsFixtures

  alias LinkShortener.Accounts.User

  @create_attrs %{
    email: "user@mail.com",
    password: "some password"
  }

  @update_attrs %{
    email: "some updated email",
    password: "some updated password"
  }

  @invalid_password_attrs %{
    email: "user@mail.com",
    password: ""
  }

  @invalid_attrs %{
    email: nil,
    encrypted_password: nil
  }

  setup %{conn: conn} do
    {:ok, conn: put_req_header(conn, "accept", "application/json")}
  end

  describe "create user with sign up" do
    test "renders user when data is valid", %{conn: conn} do
      conn = post(conn, ~p"/api/v1/users/sign-up", user: @create_attrs)
      assert %{
	"email" => "user@mail.com",
	"token" => token
      } = json_response(conn, 201)
    end

    test "renders errors when data is invalid", %{conn: conn} do
      conn = post(conn, ~p"/api/v1/users/sign-up", user: @invalid_attrs)
      assert json_response(conn, 422)["errors"] != %{}
    end
  end

  describe "user sign in" do
    setup [:create_user]

    test "renders user when data is valid", %{conn: conn} do
      conn = post(conn, ~p"/api/v1/users/sign-in", @create_attrs)

      assert %{
	"email" => email,
	"token" => token,
      } = json_response(conn, 201)
    end

    test "renders errors when data is invalid", %{conn: conn} do
      conn = post(conn, ~p"/api/v1/users/sign-in", @invalid_password_attrs)
      assert %{
	"errors" => %{"detail" => "Unauthorized"}
      } = json_response(conn, 401)
    end
  end

  describe "user signs out" do
    setup [:create_user]

    setup %{conn: conn} do
      %{token: token} = create_user_token()

      conn = conn
      |> put_req_header("accept", "application/json")
      |> put_req_header("authorization", "Bearer #{token}")

      {:ok, conn: conn, token: token}
    end

    test "renders the message and the token if successfully sign out", %{conn: conn, token: token} do
      conn = post(conn, ~p"/api/v1/users/sign-out", %{})

      assert %{
	"message" => "Successfully sign out",
	"token" => token,
      } = json_response(conn, 200)
    end

    test "renders errors if the token is invalid after revoke", %{conn: conn, token: token} do
      # revoking
      conn = post(conn, ~p"/api/v1/users/sign-out", %{})
      # second revoking
      conn = post(conn, ~p"/api/v1/users/sign-out", %{})

      assert %{
	"error" => "invalid_token"
      } = json_response(conn, 401)
    end
  end

  defp create_user_token() do
    token = user_token_fixture()
    %{token: token}
  end

  defp create_user(_) do
    user = user_fixture(@create_attrs)
    %{user: user}
  end
end
